
/******************************************************************************
 * Copyright 2020 The siasun Transport Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/
#ifndef ROBOT_ROBOT_DECISION_H
#define ROBOT_ROBOT_DECISION_H



#include "../common/robotapp.h"
#include "ros/ros.h"
#include "../common/sqlite3helper.h"

namespace robot {
namespace decision {

#define ROBOT_SUCCESS     0
#define ROBOT_FAILTURE   -1

/**
* @class Decision
*
* @brief decision module main class.
* 本类是决策类
* It collect the control data from other model and send decison message to other model.
*/

class Decision: public robot::common::RobotApp{

public:
    Decision(){

    }
    /**
    * decision节点名称
    */
    std::string Name() const override;

    /**
    * decision模块初始化
    * @brief module initialization function
    * @return initialization status
    */
    int Init() override;

    /**
    * decision模块启动
    * @brief module start function
    * @return start status
    */
    int Start() override;

    /**
    * decision模块的停止
    * @brief module stop function.
    * @return start status
    */
    void Stop() override;

    /**
    * @brief decision模块的数据库初始化，如果相关表不存在，则建立相关的表
    * @brief 相关的表:action_directive,location_map,location_rule_info
    * @return 0失败
    */
    int InitDB();

    /**
    * @brief 向action_firective表中插入一条
    * @return 0失败
    */
    int InsertActionDirective(const char* ruleName,
                              int iDirectiveID,
                              const char* directiveInfo,
                              int iDirectiveResult);

    /**
    * @brief 向location_map表中插入一条
    * @return 0失败
    */
    int InsertLocationMap(float fLocationLon1,
                          float fLocationLat1,
                          float fLocationLon2,
                          float fLocationLat2,
                          float fLocationX1,
                          float fLocationY1,
                          const char * mapId ,
                          float fLocationX2,
                          float fLocationY2);

    /**
    * @brief 向location_rule_info表中插入一条
    * @return 0失败
    */
    int InsertLocationRuleInfo(const char * ruleName,
                               int iLocationID,
                               double dLocationLon,
                               double dLocationLat,
                               double dLocationHeight,
                               float fLocationSpeed
                               );

    /**
    * @brief 在action_firective表中,
    * @brief 删除rule_name的记录
    * @return 0失败
    */
    int DeleteActionDirective(const char* ruleName);

    /**
    * @brief 在action_firective表中,
    * @brief 删除directive_id的记录
    * @return 0失败
    */
    int DeleteActionDirective(int iDirectiveID);

    /**
    * @brief 在location_map表中,
    * @brief 删除map_id的记录
    * @return 0失败
    */
    int DeleteLocationMap(const char* mapID);

    /**
    * @brief 在location_rule_info表中,
    * @brief 删除rule_name的记录
    * @return 0失败
    */
    int DeleteLocationRuleInfo(const char* RuleName);

    /**
    * @brief 在location_rule_info表中,
    * @brief 删除rule_name的记录
    * @return 0失败
    */
    int DeleteLocationRuleInfo(int iLocationID);

};


}
}
#endif // ROBOT_ROBOT_DECISION_H
